%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating IRFs (shock minus control) from the toy model in 
% "HH Debt Overhang and MP Transmission"
% by Sami Alpanda and Sarah Zubairy (Sep. 2016; revised Apr. 2018)
%
% This version has normally-distributed initial debt
% To isolate interest rate channel, set rho_qR = 0
% Baseline: d0 = d_bar, whereas high initial debt: d0 = d_bar*1.2
% To compare baseline and high debt IRFs, run_IRF_comparison.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
clc;

format long

addpath C:\ITD\Research\Programs_Manuals\CompEcon\CEtools;

%_________________________________________________________________________
%% Settings
adj_rate = 0; %1; %                 %1: adj-rate loans; 0: fixed-rate loans
fixed_rate_no_refi = 0; %0; %       %1: fixed-rate no refi, 0: fixed-rate with refi

int_only = 0;   %1; %               %0: baseline with home equity channel; 1: interest rate channel only    
rate_increase = 0;  %1;             %1: rate increase, 0: rate cut
rate_magnitude = 0.0025;            %0.01; %magnitude of the rate change

high_debt = 1; %1;  %               %0: baseline with initial debt at st-st; 1: high initial debt
high_debt_percentage = 1.2;         %gross increase in debt relative to baseline in the high-debt case

pie_effect = 0;  %1;                %0: baseline with no debt deflation; 1: with debt deflation effect 

n_periods = 20;                     %length of IRFs
Rf_periods = 1000;                  %length of R to calculate Rf

%_________________________________________________________________________
%% Model parameters
kappa = 0.0125;      %0.01;      %amortization rate
phi = 0.9;          %0.85;         %regulatory LTV ratio
gamma_bar =0.05;      %1;      % maximum extraction rate out of home equity

%shocks
rho_R = 0.85;
rho_q = 0.5;
rho_y = 0.95;
rho_pie = 0.5;

if int_only==1;
    rho_qR = 0;
else
    rho_qR = 4;
end
rho_yR = 0;     %4;         %
if pie_effect==1;
    rho_pieR = 0.2;
else
    rho_pieR = 0;
end

%steady-states
pie_bar = 1.005;
q_bar = 1;
y_bar = 1;
qhy_bar = 4;
R_bar = 0.01;

h_bar = qhy_bar*y_bar/q_bar;
Rm_bar = R_bar;
dy_bar = phi*qhy_bar;
ly_bar = (1 - (1-kappa)/pie_bar)*dy_bar;
cy_bar = 1 + ly_bar - (Rm_bar+kappa)/pie_bar*dy_bar;

d_bar = dy_bar*y_bar;
l_bar = ly_bar*y_bar;
c_bar = cy_bar*y_bar;

%_________________________________________________________________________
%% Initial conditions

%initial debt uniformly distributed
sigma_d = 0.1*d_bar;
if high_debt==1
    %d_init = (d_bar*high_debt_percentage-3*sigma_d : 0.001 : d_bar*high_debt_percentage+3*sigma_d)';
    d_init = (0.8*d_bar*high_debt_percentage : 0.001 : 1.2*d_bar*high_debt_percentage)';
else
    %d_init = (d_bar-3*sigma_d : 0.001 : d_bar+3*sigma_d)';
    d_init = (0.8*d_bar : 0.001 : 1.2*d_bar)';
end
no_d = size(d_init,1);
d_init_weight = (1/no_d)*ones(no_d,1);

% %initial debt normally distributed (quadrature nodes and weights)
% d_threshold = pie_bar*phi/(1-kappa)*q_bar*h_bar;
% if high_debt==1
%     %mu_d = d_threshold*high_debt_percentage;
%     mu_d = d_bar*high_debt_percentage;
% else
%     %mu_d = d_threshold;
%     mu_d = d_bar;
% end
% sigma_d = 0.1*d_bar;
% no_d = 50;
% [d_init, d_init_weight] = qnwnorm(no_d, mu_d, sigma_d^2);

l0 = l_bar;         %adjust when adjusting d? Not necessary.
q0 = q_bar;
y0 = y_bar;
pie0 = pie_bar;
R0 = R_bar;
Rm0 = Rm_bar;

%_________________________________________________________________________
%% Initialize time series
q_control = zeros(n_periods,no_d);
y_control = zeros(n_periods,no_d);
pie_control = zeros(n_periods,no_d);
capphi_control = zeros(n_periods,no_d);
gamma_control = zeros(n_periods,no_d);
l_control = zeros(n_periods,no_d);
d_control = zeros(n_periods,no_d);
c_control = zeros(n_periods,no_d);
Rm_control = zeros(n_periods,no_d);
Rf_control = zeros(n_periods,no_d);
HE_control = zeros(n_periods,no_d);
R_control = zeros(n_periods,no_d);

q_shock = zeros(n_periods,no_d);
y_shock = zeros(n_periods,no_d);
pie_shock = zeros(n_periods,no_d);
capphi_shock = zeros(n_periods,no_d);
gamma_shock = zeros(n_periods,no_d);
l_shock = zeros(n_periods,no_d);
d_shock = zeros(n_periods,no_d);
c_shock = zeros(n_periods,no_d);
Rm_shock = zeros(n_periods,no_d);
Rf_shock = zeros(n_periods,no_d);
HE_shock = zeros(n_periods,no_d);
R_shock = zeros(n_periods,no_d);

R_control_vec = zeros(Rf_periods,1);
R_shock_vec = zeros(Rf_periods,1);
weight_vec = zeros(1,Rf_periods);

%_________________________________________________________________________
%% Simulations (control and shock) for each initial debt level
R_control_vec(1) = (1-rho_R)*R_bar + rho_R*R0;
if rate_increase == 1;
    R_shock_vec(1) = (1-rho_R)*R_bar + rho_R*R0 + rate_magnitude;
else
    R_shock_vec(1) = (1-rho_R)*R_bar + rho_R*R0 - rate_magnitude;
end
weight_vec(1,1) = 1;
for ii = 2:Rf_periods
    R_control_vec(ii) = (1-rho_R)*R_bar + rho_R*R_control_vec(ii-1);
    R_shock_vec(ii) = (1-rho_R)*R_bar + rho_R*R_shock_vec(ii-1);
    weight_vec(1,ii) = (1-kappa)^(ii-1); 
end


for ii = 1:no_d
    d0 = d_init(ii);

    %% control simulation
    %period 1
    R_control(1,ii) = R_control_vec(1);
    if adj_rate == 1;
        Rf_control(1,ii) = R_control(1,ii);
    else
        Rf_control(1,ii) = kappa*weight_vec*R_control_vec;
    end
    q_control(1,ii) = exp( (1-rho_q)*log(q_bar) + rho_q*log(q0) - rho_qR*(R0-R_bar) );
    y_control(1,ii) = exp( (1-rho_y)*log(y_bar) + rho_y*log(y0) - rho_yR*(R0-R_bar) );
    pie_control(1,ii) = exp( (1-rho_pie)*log(pie_bar) + rho_pie*log(pie0) - rho_pieR*(R0-R_bar) );
    HE_control(1,ii) = phi*q_control(1,ii)*h_bar - (1-kappa)/pie_control(1,ii)*d0;
    gamma_control(1,ii) = (HE_control(1,ii) > 0);
    if adj_rate == 1;
        capphi_control(1,ii) = 1;
    elseif fixed_rate_no_refi == 1
        capphi_control(1,ii) = 0;
    else
        capphi_control(1,ii) = (HE_control(1,ii) > 0) & (Rf_control(1,ii) < Rm0); % -1.0e-4);
    end
    l_control(1,ii) = gamma_control(1,ii)*gamma_bar*HE_control(1,ii);
    d_control(1,ii) = (1-kappa)/pie_control(1,ii)*d0 + l_control(1,ii);
    c_control(1,ii) = y_bar + l_control(1,ii) - (Rm0+kappa)/pie_control(1,ii)*d0;
    Rm_control(1,ii) = (1-capphi_control(1,ii))*(1-kappa)/pie_control(1,ii)*d0/d_control(1,ii)*Rm0 + (l_control(1,ii)/d_control(1,ii) + capphi_control(1,ii)*(1-kappa)/pie_control(1,ii)*d0/d_control(1,ii))*Rf_control(1,ii);
    
    for t = 2:n_periods
        R_control(t,ii) = R_control_vec(t);   
        if adj_rate == 1;
            Rf_control(t,ii) = R_control(t,ii);
        else
            Rf_control(t,ii) = kappa*weight_vec(1,1:Rf_periods-t+1)*R_control_vec(t:Rf_periods);
        end
        q_control(t,ii) = exp( (1-rho_q)*log(q_bar) + rho_q*log(q_control(t-1,ii)) - rho_qR*(R_control(t-1,ii)-R_bar) );
        y_control(t,ii) = exp( (1-rho_y)*log(y_bar) + rho_y*log(y_control(t-1,ii)) - rho_yR*(R_control(t-1,ii)-R_bar) );
        pie_control(t,ii) = exp( (1-rho_pie)*log(pie_bar) + rho_pie*log(pie_control(t-1,ii)) - rho_pieR*(R_control(t-1,ii)-R_bar) );
        HE_control(t,ii) = phi*q_control(t,ii)*h_bar - (1-kappa)/pie_control(t,ii)*d_control(t-1,ii);
        gamma_control(t,ii) = (HE_control(t,ii) > 0);
        if adj_rate == 1;
            capphi_control(t,ii) = 1;
        elseif fixed_rate_no_refi == 1
            capphi_control(t,ii) = 0;
        else
            capphi_control(t,ii) = (HE_control(t,ii) > 0) & (Rf_control(t,ii) < Rm_control(t-1,ii)); % -1.0e-4);
        end
        l_control(t,ii) = gamma_control(t,ii)*gamma_bar*HE_control(t,ii);
        d_control(t,ii) = (1-kappa)/pie_control(t,ii)*d_control(t-1,ii) + l_control(t,ii);
        c_control(t,ii) = y_bar + l_control(t,ii) - (Rm_control(t-1,ii)+kappa)/pie_control(t,ii)*d_control(t-1,ii);
        Rm_control(t,ii) = (1-capphi_control(t,ii))*(1-kappa)/pie_control(t,ii)*d_control(t-1,ii)/d_control(t,ii)*Rm_control(t-1,ii) + (l_control(t,ii)/d_control(t,ii) + capphi_control(t,ii)*(1-kappa)/pie_control(t,ii)*d_control(t-1,ii)/d_control(t,ii))*Rf_control(t,ii);
    end

    %% shock simulation 
    %period 1
    R_shock(1,ii) = R_shock_vec(1);
    if adj_rate == 1;
        Rf_shock(1,ii) = R_shock(1,ii);
    else
        Rf_shock(1,ii) = kappa*weight_vec*R_shock_vec;
    end
    q_shock(1,ii) = exp( (1-rho_q)*log(q_bar) + rho_q*log(q0) - rho_qR*(R0-R_bar) );
    y_shock(1,ii) = exp( (1-rho_y)*log(y_bar) + rho_y*log(y0) - rho_yR*(R0-R_bar) );
    pie_shock(1,ii) = exp( (1-rho_pie)*log(pie_bar) + rho_pie*log(pie0) - rho_pieR*(R0-R_bar) );
    HE_shock(1,ii) = phi*q_shock(1,ii)*h_bar - (1-kappa)/pie_shock(1,ii)*d0;
    gamma_shock(1,ii) = (HE_shock(1,ii) > 0);
    if adj_rate == 1;
        capphi_shock(1,ii) = 1;
    elseif fixed_rate_no_refi == 1
        capphi_shock(1,ii) = 0;
    else
        capphi_shock(1,ii) = (HE_shock(1,ii) > 0) & (Rf_shock(1,ii) < Rm0); % -1.0e-4);
    end
    l_shock(1,ii) = gamma_shock(1,ii)*gamma_bar*HE_shock(1,ii);
    d_shock(1,ii) = (1-kappa)/pie_shock(1,ii)*d0 + l_shock(1,ii);
    c_shock(1,ii) = y_bar + l_shock(1,ii) - (Rm0+kappa)/pie_shock(1,ii)*d0;
    Rm_shock(1,ii) = (1-capphi_shock(1,ii))*(1-kappa)/pie_shock(1,ii)*d0/d_shock(1,ii)*Rm0 + (l_shock(1,ii)/d_shock(1,ii) + capphi_shock(1,ii)*(1-kappa)/pie_shock(1,ii)*d0/d_shock(1,ii))*Rf_shock(1,ii);
    
    for t = 2:n_periods
        R_shock(t,ii) = R_shock_vec(t);   
        if adj_rate == 1;
            Rf_shock(t,ii) = R_shock(t,ii);
        else
            Rf_shock(t,ii) = kappa*weight_vec(1,1:Rf_periods-t+1)*R_shock_vec(t:Rf_periods);
        end
        q_shock(t,ii) = exp( (1-rho_q)*log(q_bar) + rho_q*log(q_shock(t-1,ii)) - rho_qR*(R_shock(t-1,ii)-R_bar) );
        y_shock(t,ii) = exp( (1-rho_y)*log(y_bar) + rho_y*log(y_shock(t-1,ii)) - rho_yR*(R_shock(t-1,ii)-R_bar) );
        pie_shock(t,ii) = exp( (1-rho_pie)*log(pie_bar) + rho_pie*log(pie_shock(t-1,ii)) - rho_pieR*(R_shock(t-1,ii)-R_bar) );
        HE_shock(t,ii) = phi*q_shock(t,ii)*h_bar - (1-kappa)/pie_shock(t,ii)*d_shock(t-1,ii);
        gamma_shock(t,ii) = (HE_shock(t,ii) > 0);
        if adj_rate == 1;
            capphi_shock(t,ii) = 1;
        elseif fixed_rate_no_refi == 1
            capphi_shock(t,ii) = 0;
        else
            capphi_shock(t,ii) = (HE_shock(t,ii) > 0) & (Rf_shock(t,ii) < Rm_shock(t-1,ii)); % -1.0e-4);
        end
        l_shock(t,ii) = gamma_shock(t,ii)*gamma_bar*HE_shock(t,ii);
        d_shock(t,ii) = (1-kappa)/pie_shock(t,ii)*d_shock(t-1,ii) + l_shock(t,ii);
        c_shock(t,ii) = y_bar + l_shock(t,ii) - (Rm_shock(t-1,ii)+kappa)/pie_shock(t,ii)*d_shock(t-1,ii);
        Rm_shock(t,ii) = (1-capphi_shock(t,ii))*(1-kappa)/pie_shock(t,ii)*d_shock(t-1,ii)/d_shock(t,ii)*Rm_shock(t-1,ii) + (l_shock(t,ii)/d_shock(t,ii) + capphi_shock(t,ii)*(1-kappa)/pie_shock(t,ii)*d_shock(t-1,ii)/d_shock(t,ii))*Rf_shock(t,ii);
    end
end

%_________________________________________________________________________
%% Aggregating across different initial debt levels
q_control = q_control*d_init_weight;
y_control = y_control*d_init_weight;
pie_control = pie_control*d_init_weight;
capphi_control = capphi_control*d_init_weight;
gamma_control = gamma_control*d_init_weight;
l_control = l_control*d_init_weight;
d_control = d_control*d_init_weight;
c_control = c_control*d_init_weight;
Rm_control = Rm_control*d_init_weight;
Rf_control = Rf_control*d_init_weight;
HE_control = HE_control*d_init_weight;
R_control = R_control*d_init_weight;

q_shock = q_shock*d_init_weight;
y_shock = y_shock*d_init_weight;
pie_shock = pie_shock*d_init_weight;
capphi_shock = capphi_shock*d_init_weight;
gamma_shock = gamma_shock*d_init_weight;
l_shock = l_shock*d_init_weight;
d_shock = d_shock*d_init_weight;
c_shock = c_shock*d_init_weight;
Rm_shock = Rm_shock*d_init_weight;
Rf_shock = Rf_shock*d_init_weight;
HE_shock = HE_shock*d_init_weight;
R_shock = R_shock*d_init_weight;

%_________________________________________________________________________
%% IRFs (shock minus control)
l_IRF = l_shock - l_control;
d_IRF = d_shock - d_control;
c_IRF = c_shock - c_control;
R_IRF = R_shock - R_control;
Rf_IRF = Rf_shock - Rf_control;
Rm_IRF = Rm_shock - Rm_control;
HE_IRF = HE_shock - HE_control;
q_IRF = q_shock - q_control;
y_IRF = y_shock - y_control;
pie_IRF = pie_shock - pie_control;

IRF = [l_IRF d_IRF c_IRF R_IRF Rf_IRF Rm_IRF HE_IRF q_IRF y_IRF pie_IRF gamma_shock gamma_control capphi_shock capphi_control];
if high_debt==1
    save('IRF_R_highdebt','IRF')
elseif rate_increase == 1;
    save('IRF_R_rateincrease','IRF')
else
    save('IRF_R_baseline','IRF')
end

%________________________________________________________________________
%% Plots
time = (0:1:n_periods-1)';
line0 = zeros(n_periods,1);

ii = 2;
jj = 5;

%Shock and control simulations
figure('name','shock and control simulations')
subplot(ii,jj,1)
plot(time,d_shock,'-sb',time,d_control,'-xk',time,line0,':r','LineWidth',1.5);
title('debt')
axis tight

subplot(ii, jj,2)
plot(time,l_shock,'-sb',time,l_control,'-xk',time,line0,':r','LineWidth',1.5);
title('new borrowing')
axis tight

subplot(ii,jj,3)
plot(time,q_shock,'-sb',time,q_control,'-xk',time,line0,':r','LineWidth',1.5);
title('house price')
axis tight

subplot(ii,jj,4)
plot(time,HE_shock,'-sb',time,HE_control,'-xk',time,line0,':r','LineWidth',1.5);
title('home equity')
axis tight

subplot(ii,jj,5)
plot(time,c_shock,'-sb',time,c_control,'-xk',time,line0,':r','LineWidth',1.5);
title('consumption')
axis tight

subplot(ii,jj,6)
plot(time,R_shock,'-sb',time,R_control,'-xk',time,line0,':r','LineWidth',1.5);
title('policy rate')
axis tight

subplot(ii,jj,7)
plot(time,Rf_shock,'-sb',time,Rf_control,'-xk',time,line0,':r','LineWidth',1.5);
title('fixed rate')
axis tight

subplot(ii,jj,8)
plot(time,Rm_shock,'-sb',time,Rm_control,'-xk',time,line0,':r','LineWidth',1.5);
title('average interest')
axis tight

subplot(ii,jj,9)
plot(time,gamma_shock,'-sb',time,gamma_control,'-xk',time,line0,':r','LineWidth',1.5);
title('share: extract equity')
axis tight

subplot(ii,jj,10)
plot(time,capphi_shock,'-sb',time,capphi_control,'-xk',time,line0,':r','LineWidth',1.5);
title('share: refinance')
axis tight

% subplot(ii,jj,11)
% plot(time,pie_shock,'-sb',time,pie_control,'-xk',time,line0,':r','LineWidth',1.5);
% title('inflation')
% axis tight

legend('shock','control')


% IRFs (shock minus control)
figure('name','IRFs (shock minus control)')
subplot(ii,jj,1)
plot(time,d_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('debt')
axis tight

subplot(ii, jj,2)
plot(time,l_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('new borrowing')
axis tight

subplot(ii,jj,3)
plot(time,q_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('house price')
axis tight

subplot(ii,jj,4)
plot(time,HE_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('home equity')
axis tight

subplot(ii,jj,5)
plot(time,c_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('consumption')
axis tight

subplot(ii,jj,6)
plot(time,R_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('policy rate')
axis tight

subplot(ii,jj,7)
plot(time,Rf_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('fixed rate')
axis tight

subplot(ii,jj,8)
plot(time,Rm_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('average interest')
axis tight

subplot(ii,jj,9)
plot(time,gamma_shock,'-sb',time,gamma_control,'-xk',time,line0,':r','LineWidth',1.5);
title('share: extract equity')
axis tight

subplot(ii,jj,10)
plot(time,capphi_shock,'-sb',time,capphi_control,'-xk',time,line0,':r','LineWidth',1.5);
title('share: refinance')
axis tight

% subplot(ii,jj,11)
% plot(time,pie_IRF,'-sb',time,line0,':r','LineWidth',1.5);
% title('inflation')
% axis tight

legend('shock','control')

