%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating IRFs (shock minus control) from the toy model in 
% "HH Debt Overhang and MP Transmission"
% by Sami Alpanda and Sarah Zubairy (Sep. 2016; revised Apr. 2018)
%
% This version features fixed-rate mortgages and refi choice (0,1)
% To isolate interest rate channel, set rho_qR = 0
% Baseline: d0 = d_bar, whereas high initial debt: d0 = d_bar*1.2
% To compare baseline and high debt IRFs, run_IRF_comparison.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
clc;

format long

%_________________________________________________________________________
%% Settings
adj_rate = 0; %1; %                 %1: adj-rate loans; 0: fixed-rate loans
fixed_rate_no_refi = 1; %0; %       %1: fixed-rate no refi, 0: fixed-rate with refi

int_only = 0;   %1; %               %0: baseline with home equity channel; 1: interest rate channel only    
rate_increase = 0;  %1;             %1: rate increase, 0: rate cut
rate_magnitude = 0.0025;            %0.01; %magnitude of the rate change

high_debt = 1; %1;  %               %0: baseline with initial debt at st-st; 1: high initial debt
high_debt_percentage = 1.2;         %gross increase in debt relative to baseline in the high-debt case

pie_effect = 0;  %1;                %0: baseline with no debt deflation; 1: with debt deflation effect 

n_periods = 20;                     %length of IRFs
Rf_periods = 1000;                  %length of R to calculate Rf

%_________________________________________________________________________
%% Model parameters
kappa = 0.0125;      %0.01;      %amortization rate
phi = 0.9;          %0.85;         %regulatory LTV ratio
gamma_bar = 1;      %0.05;      %maximum extraction rate out of home equity

%shocks
rho_R = 0.85;
rho_q = 0.5;
rho_y = 0.95;
rho_pie = 0.5;

if int_only==1;
    rho_qR = 0;
else
    rho_qR = 4;
end
rho_yR = 0;     %4;         %
if pie_effect==1;
    rho_pieR = 0.2;
else
    rho_pieR = 0;
end

%steady-states
pie_bar = 1.005;
q_bar = 1;
y_bar = 1;
qhy_bar = 4;
R_bar = 0.01;  %0.0075; %

h_bar = qhy_bar*y_bar/q_bar;
Rm_bar = R_bar;
dy_bar = phi*qhy_bar;
ly_bar = (1 - (1-kappa)/pie_bar)*dy_bar;
cy_bar = 1 + ly_bar - (Rm_bar+kappa)/pie_bar*dy_bar;

d_bar = dy_bar*y_bar;
l_bar = ly_bar*y_bar;
c_bar = cy_bar*y_bar;

%_________________________________________________________________________
%% Initial conditions
if high_debt==1
    d0 = d_bar*high_debt_percentage;
else
    d0 = d_bar;
end 
l0 = l_bar;         %adjust when adjusting d? Not necessary.
q0 = q_bar;
y0 = y_bar;
pie0 = pie_bar;
R0 = R_bar;
Rm0 = Rm_bar;

%_________________________________________________________________________
%% Initialize time series
q_control = zeros(n_periods,1);
y_control = zeros(n_periods,1);
pie_control = zeros(n_periods,1);
capphi_control = zeros(n_periods,1);
gamma_control = zeros(n_periods,1);
l_control = zeros(n_periods,1);
d_control = zeros(n_periods,1);
c_control = zeros(n_periods,1);
Rm_control = zeros(n_periods,1);
Rf_control = zeros(n_periods,1);
HE_control = zeros(n_periods,1);
R_control = zeros(n_periods,1);

q_shock = zeros(n_periods,1);
y_shock = zeros(n_periods,1);
pie_shock = zeros(n_periods,1);
capphi_shock = zeros(n_periods,1);
gamma_shock = zeros(n_periods,1);
l_shock = zeros(n_periods,1);
d_shock = zeros(n_periods,1);
c_shock = zeros(n_periods,1);
Rm_shock = zeros(n_periods,1);
Rf_shock = zeros(n_periods,1);
HE_shock = zeros(n_periods,1);
R_shock = zeros(n_periods,1);

R_control_vec = zeros(Rf_periods,1);
R_shock_vec = zeros(Rf_periods,1);
weight_vec = zeros(1,Rf_periods);

%_________________________________________________________________________
%% Simulations control
R_control_vec(1) = (1-rho_R)*R_bar + rho_R*R0;
weight_vec(1,1) = 1;
for ii = 2:Rf_periods
    R_control_vec(ii) = (1-rho_R)*R_bar + rho_R*R_control_vec(ii-1);
    weight_vec(1,ii) = (1-kappa)^(ii-1); 
end

%period 1
R_control(1) = R_control_vec(1);
if adj_rate == 1;
    Rf_control(1) = R_control(1);
else
    Rf_control(1) = kappa*weight_vec*R_control_vec;
end
q_control(1) = exp( (1-rho_q)*log(q_bar) + rho_q*log(q0) - rho_qR*(R0-R_bar) );
y_control(1) = exp( (1-rho_y)*log(y_bar) + rho_y*log(y0) - rho_yR*(R0-R_bar) );
pie_control(1) = exp( (1-rho_pie)*log(pie_bar) + rho_pie*log(pie0) - rho_pieR*(R0-R_bar) );
HE_control(1) = phi*q_control(1)*h_bar - (1-kappa)/pie_control(1)*d0;
gamma_control(1) = (HE_control(1) > 0);
if adj_rate == 1;
    capphi_control(1) = 1;
elseif fixed_rate_no_refi == 1
    capphi_control(1) = 0;
else
    capphi_control(1) = (HE_control(1) > 0) & (Rf_control(1) < Rm0 -1.0e-7);
end
l_control(1) = gamma_control(1)*gamma_bar*HE_control(1);
d_control(1) = (1-kappa)/pie_control(1)*d0 + l_control(1);
c_control(1) = y_bar + l_control(1) - (Rm0+kappa)/pie_control(1)*d0;
Rm_control(1) = (1-capphi_control(1))*(1-kappa)/pie_control(1)*d0/d_control(1)*Rm0 + (l_control(1)/d_control(1) + capphi_control(1)*(1-kappa)/pie_control(1)*d0/d_control(1))*Rf_control(1);
  
for t = 2:n_periods
    R_control(t) = R_control_vec(t);   
    if adj_rate == 1;
        Rf_control(t) = R_control(t);
    else
        Rf_control(t) = kappa*weight_vec(1,1:Rf_periods-t+1)*R_control_vec(t:Rf_periods);
    end
    q_control(t) = exp( (1-rho_q)*log(q_bar) + rho_q*log(q_control(t-1)) - rho_qR*(R_control(t-1)-R_bar) );
    y_control(t) = exp( (1-rho_y)*log(y_bar) + rho_y*log(y_control(t-1)) - rho_yR*(R_control(t-1)-R_bar) );
    pie_control(t) = exp( (1-rho_pie)*log(pie_bar) + rho_pie*log(pie_control(t-1)) - rho_pieR*(R_control(t-1)-R_bar) );
    HE_control(t) = phi*q_control(t)*h_bar - (1-kappa)/pie_control(t)*d_control(t-1);
    gamma_control(t) = (HE_control(t) > 0);
    if adj_rate == 1;
        capphi_control(t) = 1;
    elseif fixed_rate_no_refi == 1
        capphi_control(t) = 0;
    else
        capphi_control(t) = (HE_control(t) > 0) & (Rf_control(t) < Rm_control(t-1) -1.0e-7);
    end
    l_control(t) = gamma_control(t)*gamma_bar*HE_control(t);
    d_control(t) = (1-kappa)/pie_control(t)*d_control(t-1) + l_control(t);
    c_control(t) = y_bar + l_control(t) - (Rm_control(t-1)+kappa)/pie_control(t)*d_control(t-1);
    Rm_control(t) = (1-capphi_control(t))*(1-kappa)/pie_control(t)*d_control(t-1)/d_control(t)*Rm_control(t-1) + (l_control(t)/d_control(t) + capphi_control(t)*(1-kappa)/pie_control(t)*d_control(t-1)/d_control(t))*Rf_control(t);
end

%_________________________________________________________________________
%% Simulations shock
if rate_increase == 1;
    R_shock_vec(1) = (1-rho_R)*R_bar + rho_R*R0 + rate_magnitude;
else
    R_shock_vec(1) = (1-rho_R)*R_bar + rho_R*R0 - rate_magnitude;
end
weight_vec(1,1) = 1;
for ii = 2:Rf_periods
    R_shock_vec(ii) = (1-rho_R)*R_bar + rho_R*R_shock_vec(ii-1);
    weight_vec(1,ii) = (1-kappa)^(ii-1); 
end

%period 1
R_shock(1) = R_shock_vec(1);
Rf_shock(1) = kappa*weight_vec*R_shock_vec;
if adj_rate == 1;
    Rf_shock(1) = R_shock(1);
end
q_shock(1) = exp( (1-rho_q)*log(q_bar) + rho_q*log(q0) - rho_qR*(R0-R_bar) );
y_shock(1) = exp( (1-rho_y)*log(y_bar) + rho_y*log(y0) - rho_yR*(R0-R_bar) );
pie_shock(1) = exp( (1-rho_pie)*log(pie_bar) + rho_pie*log(pie0) - rho_pieR*(R0-R_bar) );
HE_shock(1) = phi*q_shock(1)*h_bar - (1-kappa)/pie_shock(1)*d0;
gamma_shock(1) = (HE_shock(1) > 0);
if adj_rate == 1;
    capphi_shock(1) = 1;
elseif fixed_rate_no_refi == 1
    capphi_shock(1) = 0;
else
    capphi_shock(1) = (HE_shock(1) > 0) & (Rf_shock(1) < Rm0 -1.0e-7);
end
l_shock(1) = gamma_shock(1)*gamma_bar*HE_shock(1);
d_shock(1) = (1-kappa)/pie_shock(1)*d0 + l_shock(1);
c_shock(1) = y_bar + l_shock(1) - (Rm0+kappa)/pie_shock(1)*d0;
Rm_shock(1) = (1-capphi_shock(1))*(1-kappa)/pie_shock(1)*d0/d_shock(1)*Rm0 + (l_shock(1)/d_shock(1) + capphi_shock(1)*(1-kappa)/pie_shock(1)*d0/d_shock(1))*Rf_shock(1);
  
for t = 2:n_periods
    R_shock(t) = R_shock_vec(t);
    if adj_rate == 1;
        Rf_shock(t) = R_shock(t);
    else
        Rf_shock(t) = kappa*weight_vec(1,1:Rf_periods-t+1)*R_shock_vec(t:Rf_periods);
    end
    q_shock(t) = exp( (1-rho_q)*log(q_bar) + rho_q*log(q_shock(t-1)) - rho_qR*(R_shock(t-1)-R_bar) );
    y_shock(t) = exp( (1-rho_y)*log(y_bar) + rho_y*log(y_shock(t-1)) - rho_yR*(R_shock(t-1)-R_bar) );
    pie_shock(t) = exp( (1-rho_pie)*log(pie_bar) + rho_pie*log(pie_shock(t-1)) - rho_pieR*(R_shock(t-1)-R_bar) );
    HE_shock(t) = phi*q_shock(t)*h_bar - (1-kappa)/pie_shock(t)*d_shock(t-1);
    gamma_shock(t) = (HE_shock(t) > 0);
    if adj_rate == 1;
        capphi_shock(t) = 1;
    elseif fixed_rate_no_refi == 1
        capphi_shock(1) = 0;
    else
        capphi_shock(t) = (HE_shock(t) > 0) & (Rf_shock(t) < Rm_shock(t-1) -1.0e-7);
    end
    l_shock(t) = gamma_shock(t)*gamma_bar*HE_shock(t);
    d_shock(t) = (1-kappa)/pie_shock(t)*d_shock(t-1) + l_shock(t);
    c_shock(t) = y_bar + l_shock(t) - (Rm_shock(t-1)+kappa)/pie_shock(t)*d_shock(t-1);
    Rm_shock(t) = (1-capphi_shock(t))*(1-kappa)/pie_shock(t)*d_shock(t-1)/d_shock(t)*Rm_shock(t-1) + (l_shock(t)/d_shock(t) + capphi_shock(t)*(1-kappa)/pie_shock(t)*d_shock(t-1)/d_shock(t))*Rf_shock(t);
end

%_________________________________________________________________________
%% IRFs (shock minus control)
l_IRF = l_shock - l_control;
d_IRF = d_shock - d_control;
c_IRF = c_shock - c_control;
R_IRF = R_shock - R_control;
Rf_IRF = Rf_shock - Rf_control;
Rm_IRF = Rm_shock - Rm_control;
HE_IRF = HE_shock - HE_control;
q_IRF = q_shock - q_control;
y_IRF = y_shock - y_control;
pie_IRF = pie_shock - pie_control;

IRF = [l_IRF d_IRF c_IRF R_IRF Rf_IRF Rm_IRF HE_IRF q_IRF y_IRF pie_IRF gamma_shock gamma_control capphi_shock capphi_control];
if high_debt==1
    save('IRF_R_highdebt','IRF')
elseif rate_increase == 1;
    save('IRF_R_rateincrease','IRF')
else
    save('IRF_R_baseline','IRF')
end

%________________________________________________________________________
%% Plots
time = (0:1:n_periods-1)';
line0 = zeros(n_periods,1);

ii = 2;
jj = 5;

%Shock and control simulations
figure('name','shock and control simulations')
subplot(ii,jj,1)
plot(time,d_shock,'-sb',time,d_control,'-xk',time,line0,':r','LineWidth',1.5);
title('debt')
axis tight

subplot(ii, jj,2)
plot(time,l_shock,'-sb',time,l_control,'-xk',time,line0,':r','LineWidth',1.5);
title('new borrowing')
axis tight

subplot(ii,jj,3)
plot(time,q_shock,'-sb',time,q_control,'-xk',time,line0,':r','LineWidth',1.5);
title('house price')
axis tight

subplot(ii,jj,4)
plot(time,HE_shock,'-sb',time,HE_control,'-xk',time,line0,':r','LineWidth',1.5);
title('home equity')
axis tight

subplot(ii,jj,5)
plot(time,c_shock,'-sb',time,c_control,'-xk',time,line0,':r','LineWidth',1.5);
title('consumption')
axis tight

subplot(ii,jj,6)
plot(time,R_shock,'-sb',time,R_control,'-xk',time,line0,':r','LineWidth',1.5);
title('policy rate')
axis tight

subplot(ii,jj,7)
plot(time,Rf_shock,'-sb',time,Rf_control,'-xk',time,line0,':r','LineWidth',1.5);
title('fixed rate')
axis tight

subplot(ii,jj,8)
plot(time,Rm_shock,'-sb',time,Rm_control,'-xk',time,line0,':r','LineWidth',1.5);
title('average interest')
axis tight

subplot(ii,jj,9)
plot(time,gamma_shock,'-sb',time,gamma_control,'-xk',time,line0,':r','LineWidth',1.5);
title('indicator: extract equity')
axis tight

subplot(ii,jj,10)
plot(time,capphi_shock,'-sb',time,capphi_control,'-xk',time,line0,':r','LineWidth',1.5);
title('indicator: refinance')
axis tight

% subplot(ii,jj,11)
% plot(time,pie_shock,'-sb',time,pie_control,'-xk',time,line0,':r','LineWidth',1.5);
% title('inflation')
% axis tight

legend('shock','control')


% IRFs (shock minus control)
figure('name','IRFs (shock minus control)')
subplot(ii,jj,1)
plot(time,d_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('debt')
axis tight

subplot(ii, jj,2)
plot(time,l_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('new borrowing')
axis tight

subplot(ii,jj,3)
plot(time,q_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('house price')
axis tight

subplot(ii,jj,4)
plot(time,HE_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('home equity')
axis tight

subplot(ii,jj,5)
plot(time,c_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('consumption')
axis tight

subplot(ii,jj,6)
plot(time,R_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('policy rate')
axis tight

subplot(ii,jj,7)
plot(time,Rf_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('fixed rate')
axis tight

subplot(ii,jj,8)
plot(time,Rm_IRF,'-sb',time,line0,':r','LineWidth',1.5);
title('average interest')
axis tight

subplot(ii,jj,9)
plot(time,gamma_shock,'-sb',time,gamma_control,'-xk',time,line0,':r','LineWidth',1.5);
title('indicator: extract equity')
axis tight

subplot(ii,jj,10)
plot(time,capphi_shock,'-sb',time,capphi_control,'-xk',time,line0,':r','LineWidth',1.5);
title('indicator: refinance')
axis tight

% subplot(ii,jj,11)
% plot(time,pie_IRF,'-sb',time,line0,':r','LineWidth',1.5);
% title('inflation')
% axis tight

legend('shock','control')

